from typing import List

from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QScrollArea, QLabel, QListWidget, QListWidgetItem, QDialog, QDialogButtonBox


class ScrollablePopup(QDialog):
    def __init__(self, question: str, lst: List[str], parent=None):
        super().__init__(parent)
        self.setMinimumSize(500, 500)

        self.label = QLabel(question)
        self.scroll = QScrollArea()
        self.scroll.setWidgetResizable(True)
        self.content = QListWidget()
        self.scroll.setWidget(self.content)

        for str in lst:
            widget = QLabel(str)
            item = QListWidgetItem(self.content)
            item.setSizeHint(widget.minimumSizeHint())
            self.content.setItemWidget(item, widget)
        self.buttons = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel, Qt.Horizontal, self)
        self.buttons.accepted.connect(self.accept)
        self.buttons.rejected.connect(self.reject)
        self.layout = QtWidgets.QVBoxLayout()

        self.layout.addWidget(self.label)
        self.layout.addWidget(self.scroll)
        self.layout.addWidget(self.buttons)

        self.setLayout(self.layout)



import json

from napari_allencell_annotator.view.annotator_view import (
    AnnotatorView,
    AnnotatorViewMode,
)
import napari

from typing import Dict, List, Optional
import csv


class AnnotatorController:
    """
    A class used to control the model and view for annotations.

    Inputs
    ----------
    viewer : napari.Viewer
        a napari viewer where the plugin will be used

    Methods
    -------
    set_annot_json_data(dct : Dict[str, Dict])
        Sets annotation data dictionary.

    set_csv_name(name : str)
        Sets csv file name for writing.

    start_viewing()
        Changes view to VIEW mode and render annotations.

    stop_viewing()
        Changes view to ADD mode, resets annotations, and clears annotation json data.

    start_annotating(num_images: int, dct: Dict[str, List[str]])
        Changes annotation view to annotating mode.

    stop_annotating()
        Resets values from annotating and changes mode to VIEW.

    set_curr_img(curr_img_dict : Dict[str, str])
        Sets the current image and adds the image to annotations_dict.

    record_annotations(prev_img: str)
        Adds the outgoing image's annotation values to the files_and_annots.

    read_json(file_path : str)
        Reads a json file into a dictionary and sets annot_json_data.

    read_csv(file_path : str)
        Reads the first line of a csv file into a dictionary and sets annot_json_data.

    write_to_csv()
        Writes header and annotations to the csv file.
    """

    def __init__(self, viewer: napari.Viewer):

        # dictionary of json info:
        self.annot_json_data: Dict[str, Dict] = None
        # open in view mode
        self.view: AnnotatorView = AnnotatorView(viewer)

        self.view.show()
        # {'File Path' : path, 'Row' : str(row)}
        self.curr_img_dict: Dict[str, str] = None
        self.csv_name: str = None
        # annotation dictionary maps file paths -> [file name, FMS, annot1val, annot2val, ...]
        self.files_and_annots: Dict[str, List[str]] = {}

        self.view.cancel_btn.clicked.connect(self.stop_viewing)

        self.shuffled = None

    def set_annot_json_data(self, dct: Dict[str, Dict]):
        """Set annotation data dictionary."""
        self.annot_json_data = dct

    def set_csv_name(self, name: Optional[str] = None):
        """Set csv file name for writing."""
        self.csv_name = name

    def write_json(self, file_path: str):
        """Write annotation dictionary to a file."""
        if self.annot_json_data is not None:
            json.dump(self.annot_json_data, open(file_path, "w"), indent=4)

    def start_viewing(self):
        """Change view to VIEW mode and render annotations."""
        self.view.set_mode(mode=AnnotatorViewMode.VIEW)
        self.view.render_annotations(self.annot_json_data)

    def stop_viewing(self):
        """Change view to ADD mode, reset annotations, and clear annotation json data."""
        self.view.set_mode(mode=AnnotatorViewMode.ADD)
        self.annot_json_data = None

    def start_annotating(self, num_images: int, dct: Dict[str, List[str]], shuffled: bool):
        """
        Change annotation view to annotating mode and create files_and_annots with files.

        Parameters
        ----------
        num_images : int
            The total number of images to be annotated.
        dct : Dict[str, List[str]]
            The files to be used. path -> [name, FMS]
        """
        self.files_and_annots = dct
        self.view.set_num_images(num_images)
        self.view.set_mode(mode=AnnotatorViewMode.ANNOTATE)
        self.shuffled = shuffled

    def stop_annotating(self):
        """Reset values from annotating and change mode to ADD."""
        self.record_annotations(self.curr_img_dict["File Path"])
        self.write_csv()
        self.view.set_curr_index()
        self.files_and_annots = {}
        self.view.set_num_images()
        self.view.set_mode(mode=AnnotatorViewMode.ADD)
        self.view.render_default_values()
        self.set_curr_img()
        self.set_csv_name()

    def set_curr_img(self, curr_img: Optional[Dict[str, str]] = None):
        """
        Set the current image and add the image to annotations_dict.

        Changes next button if annotating the last image.

        Parameters
        ----------
        curr_img : Dict[str, str]
            The current image {'File Path' : 'path', 'Row' : str(rownum)}
        """
        self.curr_img_dict = curr_img
        if curr_img is not None:
            path: str = curr_img["File Path"]
            # files_and_annots values are lists File Path ->[File Name, FMS, annot1val, annot2val ...]
            # if the file has not been annotated the list is just length 2 [File Name, FMS]
            if len(self.files_and_annots[path]) < 3:
                # if the image is un-annotated render the default values

                self.view.render_default_values()
            else:
                # if the image has been annotated render the values that were entered
                # dictionary list [2::] is [annot1val, annot2val, ...]
                self.view.render_values(self.files_and_annots[path][2::])
            # convert row to int
            self.view.set_curr_index(int(curr_img["Row"]))
            # if at the end disable next
            if int(curr_img["Row"]) == self.view.num_images - 1:
                self.view.next_btn.setEnabled(False)
            else:
                self.view.next_btn.setEnabled(True)
            if int(curr_img["Row"]) == 0:
                self.view.prev_btn.setEnabled(False)
            else:
                self.view.prev_btn.setEnabled(True)

    def record_annotations(self, prev_img: str):
        """
        Add the outgoing image's annotation values to the files_and_annots.

        Parameters
        ----------
        prev_img : str
            The previous image file path.
        """
        lst: List = self.view.get_curr_annots()
        self.files_and_annots[prev_img] = self.files_and_annots[prev_img][:2:] + lst

    def read_json(self, file_path: str):
        """Read a json file into a dictionary and set annot_json_data."""
        with open(file_path, "r") as f:
            self.annot_json_data: Dict[str, Dict] = json.loads(f.read())
            print("booleans are True")

    def get_annotations_csv(self, annotations: str):
        """Read the first line of a csv file into a dictionary and set annot_json_data."""

        self.annot_json_data = json.loads(annotations)
        print("hello")

    def write_csv(self):
        """write headers and file info"""
        file = open(self.csv_name, "w")
        writer = csv.writer(file)
        writer.writerow(["Shuffled:", self.shuffled])
        header: List[str] = ["Annotations:", json.dumps(self.annot_json_data)]
        writer.writerow(header)

        header = ["File Name", "File Path", "FMS"]
        for name in self.view.annots_order:
            header.append(name)
        writer.writerow(header)
        for name, lst in self.files_and_annots.items():
            writer.writerow([name] + lst)
        file.close()



from enum import Enum

from PyQt5.QtWidgets import QPushButton
from qtpy.QtWidgets import QHBoxLayout, QWidget, QFileDialog
from qtpy.QtCore import Signal
from typing import List


class FileInputMode(Enum):
    DIRECTORY = "dir"
    FILE = "file"
    CSV = "csv"
    JSONCSV = "jsoncsv"
    JSON = "json"


class FileInput(QWidget):
    """
    A file input Widget that includes a file dialog for selecting a file / directory
    and a text box to display the selected file
    inputs:
        mode (FileInputMode): files file dialog selection type to File, Directory, or Csv.
        initial_text (str): text to display in the widget before a file has been selected
    """

    file_selected = Signal(list)
    selected_file: List[str] = None

    def __init__(
        self, parent: QWidget = None, mode: FileInputMode = FileInputMode.FILE, placeholder_text: str = None,
    ):
        super().__init__(parent)
        self._mode = mode

        self._input_btn = QPushButton(placeholder_text)
        self._input_btn.clicked.connect(self._select_file)

        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self._input_btn)
        self.setLayout(layout)

    @property
    def mode(self) -> FileInputMode:
        return self._mode

    def simulate_click(self):
        """Simulate a click event to open the file dialog."""
        self._input_btn.clicked.emit()

    def toggle(self, enabled: bool):
        """Enable user to click the add file button"""
        self._input_btn.setEnabled(enabled)

    def _select_file(self):  # pragma: no-cover
        if self._mode == FileInputMode.FILE:

            file_path, _ = QFileDialog.getOpenFileNames(
                self,
                "Select a file",
                options=QFileDialog.Option.DontUseNativeDialog | QFileDialog.Option.DontUseCustomDirectoryIcons,
            )
        elif self._mode == FileInputMode.DIRECTORY:
            file_path = QFileDialog.getExistingDirectory(
                self,
                "Select a directory",
                options=QFileDialog.Option.DontUseNativeDialog | QFileDialog.Option.DontUseCustomDirectoryIcons,
            )
            if len(file_path) > 0:
                file_path = [file_path]
            else:
                file_path = None
        elif self._mode == FileInputMode.CSV:
            file_path, _ = QFileDialog.getSaveFileName(
                self,
                "Select or create a csv file",
                filter="CSV Files (*.csv)",
                options=QFileDialog.Option.DontUseNativeDialog | QFileDialog.Option.DontUseCustomDirectoryIcons,
            )
            if file_path is None or file_path == "":
                file_path = None
            else:
                file_path = [file_path]
        elif self._mode == FileInputMode.JSON:
            file_path, _ = QFileDialog.getSaveFileName(
                self,
                "Select or create a json file",
                filter="JSON Files (*.json)",
                options=QFileDialog.Option.DontUseNativeDialog | QFileDialog.Option.DontUseCustomDirectoryIcons,
            )
            if file_path is None or file_path == "":
                file_path = None
            else:
                file_path = [file_path]
        else:
            # JSONCSV
            file_path, _ = QFileDialog.getOpenFileName(
                self,
                "Select a .csv or .json file with annotations",
                filter="CSV Files (*.csv) ;; JSON (*.json)",
                options=QFileDialog.Option.DontUseNativeDialog | QFileDialog.Option.DontUseCustomDirectoryIcons,
            )
            if file_path is None or file_path == "":
                file_path = None
            else:
                file_path = [file_path]

        if file_path:
            self.selected_file = file_path
            self.file_selected.emit(file_path)



from typing import Set

from PyQt5.QtWidgets import QListWidget, QAbstractItemView
from psygnal._signal import Signal

from napari_allencell_annotator.widgets.annotation_item import AnnotationItem


class AnnotationWidget(QListWidget):
    """
    A class used to create a QListWidget for annotations.

    """

    annots_selected = Signal(bool)

    def __init__(self):
        QListWidget.__init__(self)
        self.num_checked: int = 0
        # allow drag and drop rearrangement
        self.setDragDropMode(QAbstractItemView.InternalMove)

        # TODO: styling https://blog.actorsfit.com/a?ID=01450-929cf741-2d80-418c-8a55-a52395053369

    def clear_all(self):
        """Clear all image data."""
        self.num_checked = 0
        self.clear()

    def add_new_item(self):
        """
        Adds a new Annotation Item to the list. .

        Only allows 10 items to be added.
        """
        if self.count() < 10:
            item = AnnotationItem(self)
            item.check.stateChanged.connect(lambda: self._check_evt(item))
            h = item.sizeHint().height()
            self.setMaximumHeight(h * self.count())

    def remove_item(self, item: AnnotationItem):
        """
        Remove the item.

        Params
        -------
        item: AnnotationItem
            an item to remove.
        """
        h = item.sizeHint().height()
        self.takeItem(self.row(item))
        self.setMaximumHeight(h * self.count())

    def delete_checked(self):
        """
        Delete the checked items.

        This function emits a annots_selected signal.
        """
        lst = []
        for x in range(self.count()):
            if self.item(x).check.isChecked():
                lst.append(self.item(x))
        for item in lst:
            self.remove_item(item)
        self.num_checked = 0
        self.annots_selected.emit(False)

    def _check_evt(self, item: AnnotationItem):
        """
        Update checked set and emit files_selected signal.

        Params
        -------
        item: AnnotationItem
            the item that has been checked or unchecked.
        """
        if item.check.isChecked():
            self.num_checked = self.num_checked + 1
            if self.num_checked == 1:
                self.annots_selected.emit(True)
        elif not item.check.isChecked():
            self.num_checked = self.num_checked - 1
            if self.num_checked == 0:
                self.annots_selected.emit(False)



from typing import Dict

from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (
    QMainWindow,
    QListWidget,
    QWidget,
    QLabel,
    QGridLayout,
    QPushButton,
    QDialog,
    QDialogButtonBox,
    QVBoxLayout,
    QScrollArea,
    QHBoxLayout,
)
from psygnal._signal import Signal

from napari_allencell_annotator.widgets.annotation_item import AnnotationItem
from napari_allencell_annotator.widgets.annotation_widget import AnnotationWidget


class CreateDialog(QDialog):
    """
    A class that creates up to 10 annotations in a popup dialog.

    Attributes
    ----------
    Methods
    -------
    """

    valid_annots_made = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)

        self.setWindowTitle("Create Annotations")
        self.setMinimumSize(700, 500)

        self.list = AnnotationWidget()
        self.list.add_new_item()

        label = QLabel("Create Annotations")
        label.setAlignment(Qt.AlignCenter)
        label.setFont(QFont("Arial", 15))
        self.layout = QVBoxLayout()
        self.layout.addWidget(label)

        self.scroll = QScrollArea()
        self.scroll.setWidget(self.list)
        self.scroll.setWidgetResizable(True)
        self.layout.addWidget(self.scroll, stretch=15)

        self.add = QPushButton("Add +")
        self.delete = QPushButton("Delete Selected")
        self.cancel = QPushButton("Cancel")
        self.apply = QPushButton("Apply")
        self.btns = QWidget()
        self.new_annot_dict: Dict[str, Dict] = None
        layout = QHBoxLayout()
        layout.addWidget(self.add)
        layout.addWidget(self.delete)
        sp_retain = QtWidgets.QSizePolicy()
        sp_retain.setRetainSizeWhenHidden(True)
        self.delete.setSizePolicy(sp_retain)
        self.delete.hide()
        layout.addWidget(self.cancel)
        layout.addWidget(self.apply)

        self.btns.setLayout(layout)
        self.error = QLabel()
        self.error.setStyleSheet("color: red")
        self.layout.addWidget(self.error)
        self.layout.addWidget(self.btns)

        self.setLayout(self.layout)
        self.add.clicked.connect(self.add_clicked)
        self.cancel.clicked.connect(self.reject)
        self.apply.clicked.connect(self.get_annots)
        self.valid_annots_made.connect(self.accept)
        self.delete.clicked.connect(self._delete_clicked)
        self.list.annots_selected.connect(self._show_delete)

    def _show_delete(self, checked: bool):
        """Display the delete button"""
        if checked:
            self.delete.show()
        else:
            self.delete.hide()

    def _delete_clicked(self):
        """Call list's delete checked"""
        if self.list.num_checked > 0:
            self.list.delete_checked()

    def add_clicked(self):
        """Add a new item if there are less than 9. Hide add button otherwise"""
        self.list.add_new_item()
        if self.list.count() > 9:
            self.add.hide()

    def get_annots(self):
        """
        Set annotation dictionary to annotation data in list
        and emit valid_annots_made signal if all annotations are valid.

        """

        dct: Dict[str, Dict] = {}
        valid = True
        error = ""
        # grab all items from list of annotations
        items = [self.list.item(x) for x in range(self.list.count())]
        # if all items have been deleted annotations are invalid
        if len(items) < 1:
            valid = False
            error = " Must provide at least one annotation. "

        for i in items:
            item_valid, name, sub_dct, item_error = i.get_data()
            # sub_dct is annotation data (type,default, options)
            if name in dct.keys():
                valid = False
                i.highlight(i.name)
                error = error + " No duplicate names allowed. "
            dct[name] = sub_dct
            if not item_valid:
                valid = False
                error = error + " " + item_error
                break

        # if all values were valid emit signal and set new_annot_dict
        self.error.setText(error)
        if valid:
            self.new_annot_dict = dct
            self.valid_annots_made.emit()
        else:
            self.new_annot_dict = None



import os
from pathlib import Path
from typing import List, Dict, Optional
import random

import napari

from napari_allencell_annotator.view.images_view import ImagesView
from napari_allencell_annotator.model import images_model
from napari_allencell_annotator.constants.constants import SUPPORTED_FILE_TYPES


class ImagesController:
    """
    A Main controller to integrate view and model for image annotations

    Attributes
    ----------
    view
        the GUI and napari viewer

    Methods
    -------
    load_from_csv(files : List[str], shuffled: bool)
        Adds files to file list from a csv list of file paths.

    get_files_dict() -> Dict[str,List[str]]
        Returns the file dictionary that has the current file order.

    is_supported(file_name:str)->bool
        Returns True if a file is a supported file type.

    start_annotating()
        Sets the current item.

    stop_annotating()
        Clears file widget and reset buttons.

    curr_img_dict() -> Dict[str,str]
        Returns a dictionary with the current image attributes.

    next_img()
        Sets the current image to the next in the list.

    prev_img()
        Sets the current image to the previous one in the list.

    get_num_files(self) -> int
        Returns the number of files.
    """

    def __init__(self, viewer: napari.Viewer):
        self.model: images_model = images_model
        self.view: ImagesView = ImagesView(viewer, self)
        self.view.show()
        self._connect_slots()

    def _connect_slots(self):
        """Connects signals to slots."""
        self.view.input_dir.file_selected.connect(self._dir_selected_evt)
        self.view.input_file.file_selected.connect(self._file_selected_evt)
        self.view.shuffle.clicked.connect(self._shuffle_clicked)

    def load_from_csv(self, files: List[str], shuffled: bool):
        """
        Add files to file list from a csv list of file paths.

        If the csv files are shuffled, set shuffle order and hide file names.

        Parameters
        __________
        dct : List[str]
            a list of file paths.
        shuffled: bool
            true if files are shuffled.
        """
        for name in files:
            self.view.file_widget.add_new_item(name, shuffled)
        self.view.disable_csv_image_edits()
        if shuffled:
            self.view.file_widget.set_shuff_order(self.view.file_widget.files_dict)

    def get_files_dict(self) -> (Dict[str, List[str]], bool):
        """
        Return the file dictionary that has the current file order.

        Returns
        ----------
        Dict[str,List[str]], bool
            dictionary of file info. keys in order. a boolean shuffled.
        """
        if self.view.file_widget.shuffled:
            return self.view.file_widget.shuffled_files_dict, True
        else:
            return self.view.file_widget.files_dict, False

    def _shuffle_clicked(self, checked: bool):
        """
        Shuffle file order and hide file names if checked.
        Return files to original order and names if unchecked.

        Side effect: set file_widget.shuffled_files_dict to a new order dict or {} if list is unshuffled.

        Parameters
        ----------
        checked : bool
            Toggle state of the shuffle button.
        """

        files: Dict[str, List[str]] = self.view.file_widget.clear_for_shuff()
        if len(files) > 0:
            if checked:
                self.view.toggle_add(False)
                keys = list(files.keys())
                random.shuffle(keys)
                shuff_dict = {}
                for k in keys:
                    shuff_dict[k] = files[k]
                    self.view.file_widget.add_item(k, hidden=True)
                self.view.file_widget.set_shuff_order(shuff_dict)

            else:
                self.view.toggle_add(True)
                self.view.file_widget.set_shuff_order()
                for f in files.keys():
                    self.view.file_widget.add_item(f, hidden=False)

    @staticmethod
    def is_supported(file_path: str) -> bool:
        """
        Check if the provided file name is a supported file.

        This function checks if the file name extension is in
        the supported file types files.

        Parameters
        ----------
        file_name : str
            Name of the file to check.

        Returns
        -------
        bool
            True if the file is supported.
        """
        if file_path is None:
            return False
        extension = Path(file_path).suffix
        if extension in SUPPORTED_FILE_TYPES:
            return True
        else:
            return False

    def _dir_selected_evt(self, dir_list: List[str]):
        """
        Adds all files in a directory to the GUI.

        Parameters
        ----------
        dir_list : List[str]
            The input list with dir[0] holding directory name.
        """
        if dir_list is not None and len(dir_list) > 0:
            d = dir_list[0]
            if len(os.listdir(d)) < 1:
                self.view.alert("Folder is empty")
            else:
                for file in os.listdir(d):
                    file = d + "/" + file
                    if self.is_supported(file):
                        self.view.file_widget.add_new_item(file)
                    else:
                        self.view.alert("Unsupported file type:" + file)
        else:
            self.view.alert("No selection provided")

    def _file_selected_evt(self, file_list: List[str]):
        """
        Adds all selected files to the GUI.

        Parameters
        ----------
        file_list : List[str]
            The list of files
        """
        if file_list is None or len(file_list) < 1:
            self.view.alert("No selection provided")
        else:
            for file in file_list:
                if self.is_supported(file):
                    self.view.file_widget.add_new_item(file)
                else:
                    self.view.alert("Unsupported file type:" + file)

    def start_annotating(self, row: Optional[int] = 0):
        """Set current item to the first item."""
        if self.view.file_widget.count() > 0:
            self.view.file_widget.setCurrentItem(self.view.file_widget.item(row))

        else:
            self.view.alert("No files to annotate")

    def stop_annotating(self):
        """Clear file widget and reset buttons."""
        self.view.file_widget.clear_all()
        self.view.reset_buttons()

    def curr_img_dict(self) -> Dict[str, str]:
        """
         Return a dictionary with the current image File Path
         and row in the list.

        Returns
        ----------
        Dict[str,str]
            dictionary of file name and row attributes.
        """
        item = self.view.file_widget.currentItem()
        info = {
            "File Path": item.file_path,
            "Row": str(self.view.file_widget.get_curr_row()),
        }
        return info

    def next_img(self):
        """
        Set the current image to the next in the list, stop incrementing
        at the last row.
        """
        if self.view.file_widget.get_curr_row() < self.view.file_widget.count() - 1:
            self.view.file_widget.setCurrentItem(self.view.file_widget.item(self.view.file_widget.get_curr_row() + 1))

    def prev_img(self):
        """Set the current image to the previous in the list, stop at first image."""
        if self.view.file_widget.get_curr_row() > 0:
            self.view.file_widget.setCurrentItem(self.view.file_widget.item(self.view.file_widget.get_curr_row() - 1))

    def get_num_files(self) -> int:
        """
        Returns
        ----------
        int
            number of files.
        """
        return self.view.file_widget.count()



from enum import Enum
from typing import Dict, List, Optional

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (
    QWidget,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QSpinBox,
    QCheckBox,
    QComboBox,
    QGridLayout,
    QListWidget,
    QScrollArea,
    QListWidgetItem,
    QPushButton,
    QAbstractScrollArea,
    QMessageBox,
    QVBoxLayout,
    QSizePolicy,
)
from napari import Viewer
from napari_allencell_annotator.widgets.file_input import (
    FileInput,
    FileInputMode,
)


class AnnotatorViewMode(Enum):
    """
    Mode for view.

    ADD is used when there is not an annotation set selected
    VIEW is used when an annotation set has been made/selected, but annotating has not started.
    ANNOTATE is used when the image set is finalized and annotating has started.
    """

    ADD = "add"
    VIEW = "view"
    ANNOTATE = "annotate"


class AnnotatorView(QWidget):
    """
    A class used to create a view for annotations.

    Inputs
    ----------
    viewer : napari.Viewer
        a napari viewer where the plugin will be used
    mode : AnnotatorViewMode
        a mode for the view

    Methods
    -------
    mode() -> AnnotatorViewMode

    set_mode(mode: AnnotatorViewMode)

    set_num_images(num: int)
        Sets the total images to be annotated.

    set_curr_index(num: int)
        Sets the index of the currently selected image.

    reset_annotations()
        Resets annotation data to empty.

    render_default_values()
        Sets annotation widget values to default.

    render_values(vals: List):
        Sets the values of the annotation widgets to vals.

    get_curr_annots() -> List
        Returns the current annotation values in a list form.

    toggle_annots_editable()
        Enables the annotations for editing.

    render_annotations(data : Dict[str,Dict]))
        Renders GUI elements from the dictionary of annotations.

    popup(text:str) -> bool
        Pop up dialog that asks the user a question. Returns True if 'Yes' False if 'No'.
    """

    def __init__(
        self, viewer: Viewer, mode: AnnotatorViewMode = AnnotatorViewMode.ADD,
    ):
        super().__init__()
        self._mode = mode
        label = QLabel("Annotations")
        label.setAlignment(Qt.AlignCenter)
        label.setFont(QFont("Arial", 15))
        self.layout = QVBoxLayout()
        self.layout.addWidget(label)

        self.annot_list = QListWidget()
        self.annot_list.setSizeAdjustPolicy(QAbstractScrollArea.AdjustToContents)
        self.annot_list.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        self.scroll = QScrollArea()
        self.scroll.setWidget(self.annot_list)
        self.scroll.setWidgetResizable(True)
        self.scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scroll.setSizeAdjustPolicy(QAbstractScrollArea.AdjustToContents)
        self.scroll.setStyleSheet(
            """QScrollBar:vertical {
            width:10px;    
            margin: 0px 0px 0px 0px;
        }"""
        )
        style = """QScrollBar::handle:vertical {border: 0px solid red; border-radius: 
        2px;} """
        self.scroll.setStyleSheet(self.scroll.styleSheet() + style)
        self.layout.addWidget(self.scroll)

        self.num_images: int = None
        self.curr_index: int = None

        # Add widget visible in ADD mode
        self.add_widget = QWidget()
        add_layout = QHBoxLayout()
        self.create_btn = QPushButton("Create New Annotations")
        self.create_btn.setEnabled(True)
        self.import_btn = QPushButton("Import Existing Annotations (.csv or .json)")
        self.import_btn.setEnabled(True)
        self.annot_input = FileInput(mode=FileInputMode.JSONCSV, placeholder_text="Start Annotating")
        self.annot_input.toggle(False)

        add_layout.addWidget(self.create_btn, stretch=2)
        add_layout.addWidget(self.import_btn, stretch=2)
        self.add_widget.setLayout(add_layout)
        self.layout.addWidget(self.add_widget)

        # view widget visible in VIEW mode
        self.view_widget = QWidget()
        view_layout = QHBoxLayout()
        self.cancel_btn = QPushButton("Cancel")
        self.start_btn = QPushButton("Start Annotating")
        self.csv_input = FileInput(mode=FileInputMode.CSV)
        self.csv_input.toggle(False)
        self.save_json_btn = FileInput(mode=FileInputMode.JSON, placeholder_text="Save to JSON")
        self.save_json_btn.toggle(True)
        self.start_btn.setEnabled(True)

        view_layout.addWidget(self.cancel_btn, stretch=1)
        view_layout.addWidget(self.save_json_btn, stretch=1)
        view_layout.addWidget(self.start_btn, stretch=1)
        self.view_widget.setLayout(view_layout)

        # annot widget visible in ANNOTATE mode
        self.annot_widget = QWidget()
        annot_layout = QGridLayout()
        self.save_exit_btn = QPushButton("Save + Exit")

        self.prev_btn = QPushButton("< Previous")
        self.next_btn = QPushButton("Next >")
        self.next_btn.setEnabled(True)
        self.progress_bar = QLabel()
        annot_layout.addWidget(self.progress_bar, 0, 1, 1, 2)
        annot_layout.addWidget(self.save_exit_btn, 1, 0, 1, 2)
        annot_layout.addWidget(self.prev_btn, 1, 2, 1, 1)
        annot_layout.addWidget(self.next_btn, 1, 3, 1, 1)
        self.annot_widget.setLayout(annot_layout)

        self._display_mode()
        self.annotation_item_widgets: List[QWidget] = []
        self.annots_order: List[str] = []
        self.default_vals: List[str] = []
        self.setLayout(self.layout)
        self.viewer: Viewer = viewer

    @property
    def mode(self) -> AnnotatorViewMode:
        return self._mode

    def set_mode(self, mode: AnnotatorViewMode):
        self._mode = mode
        self._display_mode()

    def set_num_images(self, num: Optional[int] = None):
        """Set the total number of images to be annotated"""
        self.num_images = num

    def set_curr_index(self, num: Optional[int] = None):
        """Set the index of the currently selected image and display it on progress bar."""
        if num is not None:
            self.curr_index = num
            self.progress_bar.setText("{} of {} Images".format(self.curr_index + 1, self.num_images))

    def _reset_annotations(self):
        """Reset annotation data to empty."""
        self.annot_list.clear()
        # todo: make size policy, remove this line
        self.annot_list.setMaximumHeight(600)
        self.annotation_item_widgets = []
        self.annots_order: List[str] = []
        self.default_vals: List[str] = []

    def render_default_values(self):
        """Set annotation widget values to default."""
        # for curr index if annots exist fill else fill with default
        self.render_values(self.default_vals)

    def render_values(self, vals: List):
        """
        Set the values of the annotation widgets.

        Parameters
        ----------
        vals:List
            the values for the annotations.
        """
        for (widget, val, default) in zip(self.annotation_item_widgets, vals, self.default_vals):
            if val is None or val == "":
                val = default
            if isinstance(widget, QLineEdit):
                widget.setText(val)
            elif isinstance(widget, QSpinBox):
                widget.setValue(int(val))
            elif isinstance(widget, QCheckBox):
                if isinstance(val, str):
                    # val is a str from previous annotaiton
                    widget.setChecked(eval(val))
                else:
                    # val is bool from default vals
                    widget.setChecked(val)
            elif isinstance(widget, QComboBox):
                widget.setCurrentText(val)

    def get_curr_annots(self) -> List:
        """
        Return the current annotation values in a list.

        Returns
        ----------
        List
            a list of annotation values.
        """
        annots = []
        for i in self.annotation_item_widgets:
            value = ""
            if isinstance(i, QLineEdit):
                value = i.text()
            elif isinstance(i, QSpinBox):
                value = i.value()
            elif isinstance(i, QCheckBox):
                value = i.isChecked()
            elif isinstance(i, QComboBox):
                value = i.currentText()
            annots.append(value)
        return annots

    def toggle_annots_editable(self, editable: bool):
        """Enable the annotation widgets for editing."""
        for i in self.annotation_item_widgets:
            i.setEnabled(editable)

    def _display_mode(self):
        """Render GUI buttons visible depending on the mode."""
        item = self.layout.itemAt(self.layout.count() - 1)
        item.widget().hide()
        self.layout.removeItem(item)
        if self._mode == AnnotatorViewMode.ADD:
            self.add_widget.show()
            self._reset_annotations()
            self.layout.addWidget(self.add_widget)
        elif self._mode == AnnotatorViewMode.VIEW:
            self.save_json_btn.setEnabled(True)
            self.view_widget.show()
            self.layout.addWidget(self.view_widget)
        elif self._mode == AnnotatorViewMode.ANNOTATE:
            self.annot_widget.show()
            self.layout.addWidget(self.annot_widget)
            self.prev_btn.setEnabled(False)

    def render_annotations(self, data: Dict[str, Dict]):
        """
        Read annotation dictionary into individual annotations.

        Parameters
        ----------
        data : Dict[str, Dict]
            The dictionary of annotation names -> a dictionary of types, defaults, and options.
        """
        self.annotation_item_widgets = []

        for name in data.keys():
            self._create_annot(name, data[name])
            # todo fix: doesnt work if certain widgets are first leaves blank spot on bottom
        self.annot_list.setMaximumHeight(self.annot_list.item(0).sizeHint().height() * len(data))

    def _create_annot(self, name: str, dictn: Dict):
        """
        Create annotation widgets from dictionary entries.

        Parameters
        ----------
        name : str
            annotation name.
        dictn : Dict[str, (list,str,int,or bool)]
            annotation types and data.
        """
        widget = QWidget()
        layout = QHBoxLayout()
        label = QLabel(name)
        self.annots_order.append(name)

        layout.addWidget(label)
        annot_type: str = dictn["type"]
        if annot_type == "string":
            item = QLineEdit(dictn["default"])
        elif annot_type == "number":
            item = QSpinBox()
            item.setValue(dictn["default"])
        elif annot_type == "bool":
            item = QCheckBox()
            if dictn["default"]:
                item.setChecked(True)
        elif annot_type == "list":
            item = QComboBox()
            for opt in dictn["options"]:
                item.addItem(opt)
            item.setCurrentText(dictn["default"])
        layout.addWidget(item)
        self.default_vals.append(dictn["default"])
        self.annotation_item_widgets.append(item)
        item.setEnabled(True)
        layout.setContentsMargins(2, 12, 8, 12)
        layout.setSpacing(2)
        widget.setLayout(layout)
        list_item = QListWidgetItem(self.annot_list)
        list_item.setSizeHint(widget.minimumSizeHint())
        self.annot_list.setItemWidget(list_item, widget)

    def popup(self, text: str) -> bool:
        """
        Pop up dialog to ask the user yes or no.

        Parameters
        ----------
        text : str
            question for the message box.

        Returns
        ----------
        bool
            user input, true if 'Yes' false if 'No'

        """
        msg_box = QMessageBox()
        msg_box.setText(text)
        msg_box.setStandardButtons(QMessageBox.No | QMessageBox.Yes)
        return_value = msg_box.exec()
        if return_value == QMessageBox.Yes:
            return True
        else:
            return False



import os
from pathlib import Path

from PyQt5.QtWidgets import (
    QListWidgetItem,
    QListWidget,
    QWidget,
    QHBoxLayout,
    QLabel,
    QCheckBox,
)


class FileItem(QListWidgetItem):
    """
    A class used to create custom QListWidgetItems.

    Attributes
    ----------
    file_path: str
        a path to the file.
    Methods
    -------
    name() -> str
        returns the basename of the file.
    """

    def __init__(self, file_path: str, parent: QListWidget, hidden: bool = False):
        QListWidgetItem.__init__(self, parent)
        self._file_path = file_path
        self.widget = QWidget()
        self.layout = QHBoxLayout()
        if hidden:
            self.label = QLabel("Image " + str(parent.row(self) + 1))
        else:
            path: str = self.get_name()
            if len(path) > 28:
                path = path[0:27] + "..."
            self.label = QLabel(path)
        self.layout.addWidget(self.label, stretch=19)
        self.check = QCheckBox()
        self.check.setCheckState(False)
        self.check.setCheckable(not hidden)
        self.layout.addWidget(self.check, stretch=1)
        self.layout.addStretch()
        self.layout.setContentsMargins(2, 2, 0, 5)
        self.label.setStyleSheet(
            """
                QLabel{
                    border: 0px solid; 
                }
        """
        )
        self.widget.setLayout(self.layout)
        self.setSizeHint(self.widget.sizeHint())
        if parent is not None:
            parent.setItemWidget(self, self.widget)

    def get_name(self):
        """Return basename"""
        return Path(self._file_path).stem

    @property
    def file_path(self) -> str:
        return self._file_path

    def highlight(self):
        """highlight item"""
        self.label.setStyleSheet(
            """QLabel{
                            font-weight: bold;
                            text-decoration: underline;
                        }"""
        )

    def unhighlight(self):
        """unhighlight item"""
        self.label.setStyleSheet("""QLabel{}""")

    def __hash__(self):
        return hash(self._file_path)

    def __eq__(self, other):
        """Compares two ListItems file_path attributes"""
        if not isinstance(other, type(self)):
            return NotImplemented
        return self._file_path == other._file_path



import napari_allencell_annotator

from pathlib import Path


class Directories:
    """
    Provides safe paths to common module directories
    """

    _module_base_dir = Path(napari_allencell_annotator.__file__).parent

    @classmethod
    def get_assets_dir(cls) -> Path:
        """
        Path to the assets directory
        """
        return cls._module_base_dir / "assets"

    @classmethod
    def get_style_dir(cls) -> Path:
        """
        Path to the stylesheet directory
        """
        return cls._module_base_dir / "styles"



from PyQt5.QtWidgets import QListWidget, QAbstractItemView
from typing import Set, List, Optional, Dict

from qtpy.QtCore import Signal

from napari_allencell_annotator.widgets.file_item import FileItem


class FilesWidget(QListWidget):
    """
    A class used to create a QListWidget for files.

    Attributes
    ----------
    checked : Set[FileItem]
        a set of items that are currently checked
    files_dict : Dict[str , List[str]]
        a dictionary of file path -> [File Name, FMS]

    Methods
    -------
    clear_all()
        Clears all image data.
    clear_for_shuff() -> List[str]
        Clears the list display and returns the file_order.
    set_shuff_order(lst : List[str]
        Sets the shuffle order list.
    add_new_item(file:str)
        Adds a new file to the list and file_order.
    add_item(file: str, hidden: bool)
        Adds a file to the list, but not to the file_order.
    remove_item(item: ListItem)
        Removes the item from all attributes.
    delete_checked()
        Removes all items in checked.
    """

    files_selected = Signal(bool)
    files_added = Signal(bool)

    def __init__(self):
        QListWidget.__init__(self)
        self.checked: Set[FileItem] = set()
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        # files_dict holds all image info file path -> [file name, FMS]
        # also holds the original insertion order in .keys()
        self.files_dict: Dict[str, List[str]] = {}
        self.setCurrentItem(None)
        self._shuffled: bool = False
        # shuffle order holds the same file path -> [file name, FMS] as files_dict
        # only filled when the images have been shuffled
        # holds a new shuffled order in .keys()
        # when annotation starts if images are shuffled this order is given to annotation view
        self.shuffled_files_dict: Dict[str, List[str]] = {}

    @property
    def shuffled(self) -> bool:
        return self._shuffled

    def get_curr_row(self) -> int:
        if self.currentItem() is not None:
            return self.row(self.currentItem())
        else:
            return -1

    def clear_all(self):
        """Clear all image data."""
        self._shuffled = False
        self.checked = set()
        self.files_dict = {}
        self.shuffled_files_dict = {}
        self.setCurrentItem(None)
        self.clear()

    def set_shuff_order(self, dct: Optional[Dict[str, List[str]]] = {}):
        """Set shuffled order."""
        if len(dct) > 0:
            self._shuffled = True
        self.shuffled_files_dict = dct

    def clear_for_shuff(self) -> Dict[str, List[str]]:
        """
        Clear the list display and return the files_dict.

        This function clears all displayed, checked, and current items, but keeps the files_dict.

        Returns
        -------
        List[str]
            file_order.
        """
        self._shuffled = not self._shuffled
        self.shuffled_files_dict = {}
        self.setCurrentItem(None)
        self.checked = set()
        self.clear()
        return self.files_dict

    def add_new_item(self, file: str, shuffle: Optional[bool] = False):
        """
        Adds a new file to the list and files_dict.

        This function emits a files_added signal when this is the first file added.

        Params
        -------
        file: str
            a file path.
        """
        if file not in self.files_dict.keys():
            item = FileItem(file, self, shuffle)
            item.check.stateChanged.connect(lambda: self._check_evt(item))
            self.files_dict[file] = [item.get_name(), ""]
            if len(self.files_dict) == 1:
                self.files_added.emit(True)

    def add_item(self, file: str, hidden: bool = False):
        """
        Add a file to the list, but not to the files_dict.

        Optional hidden parameter toggles file name visibility.

        Params
        -------
        file: str
            a file path.
        hidden: bool
            file name visibility.
        """
        item = FileItem(file, self, hidden)
        item.check.stateChanged.connect(lambda: self._check_evt(item))

    def remove_item(self, item: FileItem):
        """
        Remove the item from all attributes.

        This function emits a files_added signal when the item to remove is the only item.

        Params
        -------
        item: FileItem
            an item to remove.
        """
        if item.file_path in self.files_dict.keys():
            if item == self.currentItem():
                self.setCurrentItem(None)
            self.takeItem(self.row(item))
            del self.files_dict[item.file_path]
            if len(self.files_dict) == 0:
                self.files_added.emit(False)

    def delete_checked(self):
        """
        Delete the checked items.

        This function emits a files_selected signal.
        """
        for item in self.checked:
            self.remove_item(item)
        self.checked.clear()
        self.files_selected.emit(False)

    def _check_evt(self, item: FileItem):
        """
        Update checked set and emit files_selected signal.

        Params
        -------
        item: FileItem
            the item that has been checked or unchecked.
        """
        if item.check.isChecked() and item not in self.checked:
            self.checked.add(item)
            if len(self.checked) == 1:
                self.files_selected.emit(True)
        elif not item.check.isChecked() and item in self.checked:
            self.checked.remove(item)
            if len(self.checked) == 0:
                self.files_selected.emit(False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import codecs
from setuptools import setup, find_packages


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()


requirements = [
    "napari>=0.4.9",
    "napari-plugin-engine>=0.1.4",
    "numpy",
    "aicssegmentation ~= 0.4.1",
    "magicgui >= 0.2.9",
    "aicsimageio >= 4.0.5",
]

test_requirements = [
    "black>=19.10b0",
    "codecov>=2.0.22",
    "docutils>=0.10,<0.16",
    "flake8>=3.7.7",
    "psutil>=5.7.0",
    "pytest>=4.3.0",
    "pytest-cov==2.6.1",
    "pytest-raises>=0.10",
    "pytest-qt>=3.3.0",
    "quilt3>=3.1.12",
]

dev_requirements = [
    "black>=19.10b0",
    "bumpversion>=0.5.3",
    "docutils>=0.10,<0.16",
    "flake8>=3.7.7",
    "gitchangelog>=3.0.4",
    "ipython>=7.5.0",
    "m2r>=0.2.1",
    "pytest>=4.3.0",
    "pytest-cov==2.6.1",
    "pytest-raises>=0.10",
    "pytest-runner>=4.4",
    "pytest-qt>=3.3.0",
    "quilt3>=3.1.12",
    "Sphinx>=2.0.0b1,<3",
    "sphinx_rtd_theme>=0.1.2",
    "tox>=3.5.2",
    "twine>=1.13.0",
    "wheel>=0.33.1",
]

setup_requirements = [
    "pytest-runner",
]

extra_requirements = {
    "test": test_requirements,
    "dev": dev_requirements,
    "setup": setup_requirements,
    "all": [*requirements, *test_requirements, *setup_requirements, *dev_requirements,],
}


# https://github.com/pypa/setuptools_scm
use_scm = {"write_to": "napari_allencell_annotator/_version.py"}

setup(
    name="napari-allencell-annotator",
    author="Allen Institute for Cell Science",
    url="https://github.com/aics-int/napari-allencell-annotator/",
    description="A plugin that enables annotations provided by Allen Institute for Cell Science",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    python_requires=">=3.7",
    install_requires=requirements,
    setup_requires=setup_requirements,
    test_suite="napari_allencell_annotator/_tests",
    tests_require=test_requirements,
    extras_require=extra_requirements,
    include_package_data=True,
    classifiers=[
        "Intended Audience :: Science/Research",
        "Framework :: napari",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Visualization",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: BSD License",
    ],
    # Do not edit this string manually, always use bumpversion
    # Details in CONTRIBUTING.rst
    version="0.0.4",
    zip_safe=False,
)



from typing import Tuple, Dict

from PyQt5 import QtWidgets
from PyQt5.QtWidgets import (
    QListWidgetItem,
    QListWidget,
    QWidget,
    QHBoxLayout,
    QLineEdit,
    QComboBox,
    QLabel,
    QSpinBox,
    QGridLayout,
    QCheckBox,
)


class AnnotationItem(QListWidgetItem):
    """
    A class used to create custom annotation QListWidgetItems.
    """

    def __init__(self, parent: QListWidget):
        QListWidgetItem.__init__(self, parent)
        self.widget = QWidget()
        self.layout = QGridLayout()
        name_label = QLabel("Name:")
        self.name = QLineEdit()
        self.name.setPlaceholderText("Enter name")

        type_label = QLabel("Type:")
        self.type = QComboBox()
        self.type.addItems(["text", "number", "checkbox", "dropdown"])
        self.name.setWhatsThis("name")
        self.type.setWhatsThis("type")
        self.name_widget = QWidget()
        name_layout = QHBoxLayout()
        self.check = QCheckBox()
        name_layout.addWidget(self.check)
        name_layout.addWidget(name_label)
        self.name_widget.setLayout(name_layout)
        self.layout.addWidget(self.name_widget, 0, 0, 1, 1)
        self.layout.addWidget(self.name, 0, 1, 1, 2)
        self.layout.addWidget(type_label, 0, 3, 1, 1)
        self.layout.addWidget(self.type, 0, 4, 1, 2)
        default_label = QLabel("Default:")
        self.default_text = QLineEdit()
        self.default_text.setPlaceholderText("Default Text")
        self.default_num = QSpinBox()
        self.default_num.setValue(2)
        self.default_check = QComboBox()
        self.default_check.addItems(["checked", "unchecked"])
        self.default_options_label = QLabel("Options:")
        self.default_options = QLineEdit()
        self.default_options.setPlaceholderText("Enter a comma separated list of options")

        self.default_options.setSizePolicy(QtWidgets.QSizePolicy.Policy.Ignored, QtWidgets.QSizePolicy.Policy.Preferred)
        self.default_options.setMinimumWidth(300)

        sp_retain = QtWidgets.QSizePolicy()
        sp_retain.setRetainSizeWhenHidden(True)
        self.default_options.setSizePolicy(sp_retain)
        self.default_options_label.setSizePolicy(sp_retain)

        self.layout.addWidget(default_label, 0, 6, 1, 1)
        self.layout.addWidget(self.default_text, 0, 7, 1, 2)
        self.layout.addWidget(self.default_options_label, 1, 1, 1, 1)
        self.layout.addWidget(self.default_options, 1, 2, 1, 7)
        self.default_options.hide()
        self.default_options_label.hide()

        self.layout.setContentsMargins(5, 5, 5, 15)

        self.widget.setLayout(self.layout)
        self.setSizeHint(self.widget.sizeHint())
        if parent is not None:
            parent.setItemWidget(self, self.widget)

        self.type.currentTextChanged.connect(self.type_changed)

    def type_changed(self, text: str):
        """Render the widgets which correspond to the new type"""
        default_widget = self.layout.itemAtPosition(0, 7).widget()
        default_widget.setParent(None)
        self.layout.removeWidget(default_widget)

        if text == "text":
            self.default_options.hide()
            self.default_options_label.hide()
            self.layout.addWidget(self.default_text, 0, 7, 1, 2)

        elif text == "number":
            self.default_options.hide()
            self.default_options_label.hide()
            self.layout.addWidget(self.default_num, 0, 7, 1, 2)

        elif text == "checkbox":
            self.default_options.hide()
            self.default_options_label.hide()
            self.layout.addWidget(self.default_check, 0, 7, 1, 2)
        else:
            self.default_options.show()
            self.default_options_label.show()
            self.layout.addWidget(self.default_text, 0, 7, 1, 2)

    def get_data(self) -> Tuple[bool, str, Dict, str]:
        """
        Highlight any invalid entries and return the data.

        Return True if all data is valid along with str name and a dictionary
        of annotation data.
        Return False if data is invalid, highlight the incorrect entries, and return
        an incomplete dictionary.

        Returns
        ------
        Tuple[bool, str, Dict]
            bool : True if entries are valid.
            str: name of annotation item
            Dict: annotation item data (type, default, options)
            str: error msg if applicable
        """
        # bool valid if all annotation values are in the correct format
        error = ""
        valid: bool = True
        # test if annotation name is valid
        name: str = self.name.text()
        self._unhighlight(self.name)
        if name is None or name.isspace() or len(name) == 0:
            valid = False
            self.highlight(self.name)
            error = " Invalid Name. "

        type: str = self.type.currentText()
        # dictionary of annotation type, default, options
        dct: Dict = {}

        if type == "text" or type == "dropdown":
            # grab default text entry
            default = self.default_text.text()
            if default is None or default.isspace() or len(default) == 0:
                dct["default"] = ""
            else:
                # default text exists
                dct["default"] = default
            if type == "text":
                dct["type"] = "string"
            else:
                # type is options
                # comma separate list of options
                txt2 = self.default_options.text().split(",")
                # unhighlight by default
                self._unhighlight(self.default_options)
                # if there is less than two options provided
                if txt2 is None or len(txt2) < 2:
                    valid = False
                    self.highlight(self.default_options)
                    error = error + " Must provide two dropdown options. "
                else:
                    contained: bool = False
                    if dct["default"] == "":
                        contained = True
                    for item in txt2:
                        # check each item in options
                        if item.isspace() or len(item) == 0:
                            valid = False
                            self.highlight(self.default_options)
                            error = error + " Invalid options for dropdown. "
                            break
                        else:
                            if not contained and item == dct["default"]:
                                contained = True
                    if not contained:
                        txt2.append(default)
                    dct["options"] = txt2
                    dct["type"] = "list"
        elif type == "number":
            # number defaults are required by spinbox, always valid
            dct["type"] = "number"
            dct["default"] = self.default_num.value()
        else:
            # checkbox type default required by the drop down, always valid
            dct["type"] = "bool"
            if self.default_check.currentText() == "checked":
                dct["default"] = True
            else:
                dct["default"] = False
        return valid, name, dct, error

    def highlight(self, objct: QWidget):
        objct.setStyleSheet("""QLineEdit{border: 1px solid red}""")

    def _unhighlight(self, objct: QWidget):
        objct.setStyleSheet("""QLineEdit{}""")



import csv
import os
from pathlib import Path

from PyQt5.QtWidgets import QWidget, QVBoxLayout, QDialog

from napari_allencell_annotator.controller.images_controller import ImagesController

from napari_allencell_annotator.controller.annotator_controller import AnnotatorController
from napari_allencell_annotator.widgets.create_dialog import CreateDialog
import napari
from typing import List, Dict


class MainController(QWidget):
    """
    A class used to combine/communicate between AnnotatorController and ViewController.

    Methods
    -------
    _start_annotating_clicked()
        Verifies that images are added and user wants to proceed, then opens a .csv file dialog.
    stop_annotating()
         Stops annotating in images and annotations views.
    _next_image_clicked()
        Moves to the next image for annotating.
    _prev_image_clicked()
        Moves to the previous image for annotating.
    """

    def __init__(self):
        super().__init__()
        self.napari = napari.Viewer()
        self.napari.window.qt_viewer.dockLayerList.setVisible(False)
        self.napari.window.qt_viewer.dockLayerControls.setVisible(False)
        self.layout = QVBoxLayout()
        self.images = ImagesController(self.napari)
        self.annots = AnnotatorController(self.napari)
        self.layout.addWidget(self.images.view, stretch=1)
        self.layout.addWidget(self.annots.view, stretch=2)
        self.setLayout(self.layout)
        self.show()
        self.napari.window.add_dock_widget(self, area="right")
        self._connect_slots()
        self.already_annotated: Dict[str, List] = None
        self.starting_row: int = -1

    def _connect_slots(self):
        """Connects annotator view buttons start, next, and prev to slots"""
        self.annots.view.start_btn.clicked.connect(self._start_annotating_clicked)

        self.annots.view.next_btn.clicked.connect(self._next_image_clicked)
        self.annots.view.prev_btn.clicked.connect(self._prev_image_clicked)
        self.annots.view.csv_input.file_selected.connect(self._csv_write_selected_evt)
        self.annots.view.save_exit_btn.clicked.connect(self._save_and_exit_clicked)
        self.annots.view.import_btn.clicked.connect(self._import_annots_clicked)
        self.annots.view.annot_input.file_selected.connect(self._csv_import_selected_evt)
        self.annots.view.create_btn.clicked.connect(self._create_clicked)
        self.annots.view.save_json_btn.file_selected.connect(self._json_write_selected_evt)

    def _create_clicked(self):
        """Create dialog window and start viewing on accept."""
        dlg = CreateDialog(self)
        if dlg.exec() == QDialog.Accepted:
            self.already_annotated = None
            self.annots.set_annot_json_data(dlg.new_annot_dict)
            self.annots.start_viewing()

    def _json_write_selected_evt(self, file_list: List[str]):
        """
        Set json file name for writing annotations and write the annotations.

        Ensure that all file names have .json extension and that a
        file name is selected.

        Parameters
        ----------
        file_list : List[str]
            The list containing one file name.
        """

        if file_list is None or len(file_list) < 1:
            self.images.view.alert("No selection provided")
        else:
            file_path = file_list[0]
            extension = Path(file_path).suffix
            if extension != ".json":
                file_path = file_path + ".json"
            self.annots.view.save_json_btn.setEnabled(False)
            self.annots.write_json(file_path)

    def _csv_import_selected_evt(self, file_list: List[str]):
        """
        Set csv file name for importing annotations.

        Parameters
        ----------
        file_list : List[str]
            The list containing one file name.
        """
        self.already_annotated = None
        if file_list is None or len(file_list) < 1:
            self.images.view.alert("No selection provided")
        else:
            file_path = file_list[0]
            if Path(file_path).suffix == ".json":
                self.annots.read_json(file_path)

            elif Path(file_path).suffix == ".csv":
                proceed: bool = self.annots.view.popup("Would you like to use the images in this csv?")
                file = open(file_path)

                reader = csv.reader(file)
                shuffled = next(reader)[1]
                shuffled = self.str_to_bool(shuffled)
                # annotation data header
                annts = next(reader)[1]
                # skip actual header
                next(reader)
                self.starting_row = None
                if proceed:
                    self.already_annotated = {}

                    row_num: int = 0
                    for row in reader:

                        self.already_annotated[row[0]] = row[1::]
                        if self.starting_row is None:
                            for j in row[3::]:
                                if j is None or j == "":
                                    # if there is a none value for an annotation
                                    self.starting_row = row_num
                                    break
                            row_num = row_num + 1

                    self.images.load_from_csv(self.already_annotated.keys(), shuffled)
                if self.starting_row is None:
                    self.starting_row = 0
                file.close()
                self.annots.get_annotations_csv(annts)

            self.annots.start_viewing()

    def str_to_bool(self, string):
        """
        Convert a string to a bool.

        Parameters
        ----------
        string_ : str
        default : {'raise', False}
            Default behaviour if none of the "true" strings is detected.

        Returns
        -------
        boolean : bool
        """
        if string.lower() == "true":
            return True
        elif string.lower() == "false":
            return False
        else:
            raise ValueError("The value '{}' cannot be mapped to boolean.".format(string))

    def _import_annots_clicked(self):
        """Open file widget for importing csv/json."""
        self.annots.view.annot_input.simulate_click()

    def _csv_write_selected_evt(self, file_list: List[str]):
        """
        Set csv file name for writing annotations.

        Ensure that all file names have .csv extension and that a
        file name is selected.

        Parameters
        ----------
        file_list : List[str]
            The list containing one file name.
        """
        if file_list is None or len(file_list) < 1:
            self.images.view.alert("No selection provided")
        else:
            file_path = file_list[0]
            extension = Path(file_path).suffix
            if extension != ".csv":
                file_path = file_path + ".csv"
            self.annots.set_csv_name(file_path)
            self._setup_annotating()

    def _start_annotating_clicked(self):
        """
        Verify that images are added and user wants to proceed, then
        open a .csv file dialog.

        Alert user if there are no files added.
        """
        if self.images.get_num_files() is None or self.images.get_num_files() < 1:
            self.images.view.alert("Can't Annotate Without Adding Images")
        else:
            proceed: bool = self.annots.view.popup(
                "Once annotating starts both the image set and annotations cannot be "
                "edited.\n Would "
                "you like to continue?"
            )
            if proceed:
                self.annots.view.csv_input.simulate_click()

    def _stop_annotating(self):
        """
        Stop annotating in images and annotations views.

        Display images and annots views.
        """
        if not self.images.view.file_widget.shuffled:
            self.images.view.file_widget.currentItemChanged.disconnect(self.image_selected)
        self.layout.addWidget(self.images.view, stretch=1)
        self.layout.addWidget(self.annots.view, stretch=2)
        self.images.view.show()
        self.annots.stop_annotating()
        self.images.stop_annotating()
        self.images.view.input_file.show()
        self.images.view.input_dir.show()

    def _setup_annotating(self):
        """Hide the file viewer and start the annotating process."""
        dct, shuffled = self.images.get_files_dict()
        if shuffled:
            self.layout.removeWidget(self.images.view)

            self.images.view.hide()
        if self.already_annotated is not None and len(self.already_annotated) > 0:

            self.images.start_annotating(self.starting_row)
            self.annots.start_annotating(self.images.get_num_files(), self.already_annotated, shuffled)
        else:
            self.images.start_annotating()
            self.annots.start_annotating(self.images.get_num_files(), dct, shuffled)
        self.annots.set_curr_img(self.images.curr_img_dict())
        if not shuffled:
            self.images.view.file_widget.currentItemChanged.connect(self.image_selected)
            self.images.view.input_dir.hide()
            self.images.view.input_file.hide()

    def _next_image_clicked(self):
        """
        Move to the next image for annotating.

        If the last image is being annotated, write to csv. If the second
        image is being annotated, enable previous button.
        """
        self.annots.record_annotations(self.images.curr_img_dict()["File Path"])

        self.images.next_img()
        self.annots.set_curr_img(self.images.curr_img_dict())

    def image_selected(self, current, previous):
        if previous:
            self.annots.record_annotations(previous.file_path)
        self.annots.set_curr_img(self.images.curr_img_dict())

    def _prev_image_clicked(self):
        """
        Move to the previous image for annotating.

        If the first image is being annotated, disable button.
        """
        self.annots.record_annotations(self.images.curr_img_dict()["File Path"])
        self.images.prev_img()
        self.annots.set_curr_img(self.images.curr_img_dict())

    def _save_and_exit_clicked(self):
        """Stop annotation if user confirms choice in popup."""
        proceed: bool = self.annots.view.popup("Close this session?")

        if proceed:
            self._stop_annotating()



from typing import List

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QFont

from PyQt5.QtWidgets import (
    QWidget,
    QLabel,
    QScrollArea,
    QGridLayout,
    QPushButton,
    QMessageBox,
    QDialog,
)
import napari
from aicsimageio import AICSImage, exceptions
from napari.utils.notifications import show_info

from napari_allencell_annotator.widgets.file_input import (
    FileInput,
    FileInputMode,
)
from napari_allencell_annotator.widgets.scrollable_popup import ScrollablePopup
from napari_allencell_annotator.widgets.files_widget import FilesWidget, FileItem


class ImagesView(QWidget):
    """
    A class used to create a view for image file uploading and selecting.

    Attributes
    ----------
    viewer : napari.Viewer
        a napari viewer where the plugin will be used
    ctrl : ImagesController
        a controller for the view

    Methods
    -------
    alert(alert:str)
        Displays the alert message on the napari viewer
    """

    def __init__(self, viewer: napari.Viewer, ctrl):
        """
        Parameters
        ----------
        viewer : napari.Viewer
            The napari viewer for the plugin
        ctrl : ImagesController
            The controller
        """
        super().__init__()

        self.label = QLabel()
        self.label.setAlignment(Qt.AlignCenter)
        self.label.setText("Images")

        self.label.setFont(QFont("Arial", 15))
        self.layout = QGridLayout()
        self.layout.addWidget(self.label, 0, 0, 1, 4)

        self.input_dir: FileInput
        self.input_file: FileInput

        self.input_dir = FileInput(mode=FileInputMode.DIRECTORY, placeholder_text="Add a folder...")

        self.input_file = FileInput(mode=FileInputMode.FILE, placeholder_text="Add files...")
        self.layout.addWidget(self.input_dir, 1, 0, 1, 2)
        self.layout.addWidget(self.input_file, 1, 2, 1, 2)

        self.file_widget = FilesWidget()
        self.file_widget.files_selected.connect(self._toggle_delete)
        self.file_widget.files_added.connect(self._toggle_shuffle)

        self.scroll = QScrollArea()
        self.scroll.setWidget(self.file_widget)
        self.scroll.setWidgetResizable(True)
        self.layout.addWidget(self.scroll, 2, 0, 10, 4)

        self.shuffle = QPushButton("Shuffle and Hide")
        self.shuffle.setCheckable(True)
        self.shuffle.toggled.connect(self._update_shuff_text)

        self.shuffle.setEnabled(False)

        self.delete = QPushButton("Delete Selected")
        self.delete.setEnabled(False)

        self.delete.clicked.connect(self._delete_clicked)

        self.layout.addWidget(self.shuffle, 13, 0, 1, 3)
        self.layout.addWidget(self.delete, 13, 3, 1, 1)

        self.setLayout(self.layout)

        self.file_widget.currentItemChanged.connect(self._display_img)

        self.ctrl = ctrl
        self.viewer = viewer

    def _update_shuff_text(self, checked: bool):
        """
        Update shuffle button text to reflect toggle state.

        Parameters
        ----------
        checked : bool
            Toggle state of shuffle button.
        """
        if checked:
            self.shuffle.setText("Unshuffle and Unhide")
        else:
            self.shuffle.setText("Shuffle and Hide")

    def reset_buttons(self):
        self.enable_image_edits()
        self._toggle_delete(False)
        self._toggle_shuffle(False)
        self._update_shuff_text(False)
        self.toggle_add(True)

    def _delete_clicked(self):
        if len(self.file_widget.checked) > 0:
            msg: str = "Delete these files?"
            lst: List[str] = []
            for item in self.file_widget.checked:
                lst.append("--- " + item.file_path)
            msg_box = ScrollablePopup(msg, lst)
            if msg_box.exec() == QDialog.Accepted:
                self.file_widget.delete_checked()
        else:
            self.alert("No Images Selected")

    def alert(self, alert_msg: str):
        """
        Displays an error alert on the napari viewer.

        Parameters
        ----------
        alert_msg : str
            The message to be displayed
        """
        # TODO: move to main view
        show_info(alert_msg)

    def toggle_add(self, enable: bool):
        """
        Enables add file and add directory buttons.

        Parameters
        ----------
        enable : bool
            The enable state
        """
        self.input_dir.toggle(enable)
        self.input_file.toggle(enable)

    def _toggle_delete(self, checked: bool):
        """
        Enable delete button when files are checked.

        Parameters
        ----------
        checked : bool
        """
        if checked:
            self.delete.setEnabled(True)
        elif not checked:
            self.delete.setEnabled(False)

    def _toggle_shuffle(self, files_added: bool):
        """
        Enable shuffle button when files are added.

        Parameters
        ----------
        files_added : bool
        """
        if files_added:
            self.shuffle.setEnabled(True)
        elif not files_added:
            self.shuffle.setEnabled(False)

    def enable_image_edits(self):
        """Show shuffle and delete buttons"""
        self.shuffle.show()
        self.delete.show()

    def disable_csv_image_edits(self):
        """Hide shuffle and delete buttons"""
        self.shuffle.hide()
        self.delete.hide()

    def _display_img(self, current: FileItem, previous: FileItem):
        """Display the current image in napari."""
        self.viewer.layers.clear()
        if previous is not None:
            previous.unhighlight()
        if current is not None:
            try:
                img = AICSImage(current.file_path)
                self.viewer.add_image(img.data)
                current.highlight()
            except exceptions.UnsupportedFileFormatError:
                self.alert("AICS Unsupported File Type")
            except FileNotFoundError:
                self.alert("File no longer exists")
                self.file_widget.remove_item(current)



import sys

from PyQt5.QtWidgets import QApplication

from napari_allencell_annotator.controller.main_controller import MainController


class App(QApplication):
    """
    A class used to initialize the image annotator controller.
    """

    def __init__(self, sys_argv):
        super(App, self).__init__(sys_argv)

        MainController()


if __name__ == "__main__":
    app = App(sys.argv)

    sys.exit(app.exec_())
